{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [],
   "source": [
    "puzzle_input = open(\"puzzle_input/day13.txt\").readlines()\n",
    "\n",
    "test_input = [\n",
    "    \"/> <\\\\  \",\n",
    "    \"       \",\n",
    "    \"  /<+ \\\\\",\n",
    "    \"      v\",\n",
    "    \"\\\\>+</  \",\n",
    "    \"      ^\",\n",
    "    \"  \\\\< >/\",\n",
    "]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "part one: 123 18\n"
     ]
    }
   ],
   "source": [
    "class Elf(object):\n",
    "    def __init__(self, x, y, elf_char):\n",
    "        self.x = x\n",
    "        self.y = y\n",
    "        self.dir_x = 0\n",
    "        self.dir_y = 0\n",
    "        self.intersections = 0\n",
    "        self.active = True\n",
    "        if elf_char == \"<\":\n",
    "            self.dir_x = -1\n",
    "        elif elf_char == \">\":\n",
    "            self.dir_x = 1\n",
    "        elif elf_char == \"^\":\n",
    "            self.dir_y = -1\n",
    "        elif elf_char == \"v\":\n",
    "            self.dir_y = 1\n",
    "        else:\n",
    "            raise Exception(\"INVALID ELF:\", elf_char)\n",
    "    \n",
    "    def any_collisions(self, elves):\n",
    "        return len([ elf for elf in elves if elf.x == self.x and elf.y == self.y ]) != 0\n",
    "    \n",
    "    def corner(self, turn_type):\n",
    "        tmp = self.dir_x\n",
    "        if turn_type == \"/\":\n",
    "            self.dir_x = -self.dir_y\n",
    "            self.dir_y = -tmp\n",
    "        else:\n",
    "            self.dir_x = self.dir_y\n",
    "            self.dir_y = tmp\n",
    "            \n",
    "    def intersection(self):\n",
    "        intersection_mod3 = self.intersections % 3\n",
    "        if intersection_mod3 == 0:\n",
    "            self.turn_left()\n",
    "        elif intersection_mod3 == 1:\n",
    "            pass\n",
    "        elif intersection_mod3 == 2:\n",
    "            self.turn_right()\n",
    "        self.intersections += 1\n",
    "    \n",
    "    def turn_left(self):\n",
    "        if self.dir_x == 1 and self.dir_y == 0:\n",
    "            self.dir_x = 0\n",
    "            self.dir_y = -1\n",
    "        elif self.dir_x == 0 and self.dir_y == -1:\n",
    "            self.dir_x = -1\n",
    "            self.dir_y = 0\n",
    "        elif self.dir_x == -1 and self.dir_y == 0:\n",
    "            self.dir_x = 0\n",
    "            self.dir_y = 1\n",
    "        elif self.dir_x == 0 and self.dir_y == 1:\n",
    "            self.dir_x = 1\n",
    "            self.dir_y = 0\n",
    "            \n",
    "    def turn_right(self):\n",
    "        if self.dir_x == 1 and self.dir_y == 0:\n",
    "            self.dir_x = 0\n",
    "            self.dir_y = 1\n",
    "        elif self.dir_x == 0 and self.dir_y == 1:\n",
    "            self.dir_x = -1\n",
    "            self.dir_y = 0\n",
    "        elif self.dir_x == -1 and self.dir_y == 0:\n",
    "            self.dir_x = 0\n",
    "            self.dir_y = -1\n",
    "        elif self.dir_x == 0 and self.dir_y == -1:\n",
    "            self.dir_x = 1\n",
    "            self.dir_y = 0\n",
    "\n",
    "    def step(self):\n",
    "        self.x += self.dir_x\n",
    "        self.y += self.dir_y\n",
    "\n",
    "def split(str):\n",
    "    return [ c for c in str ]\n",
    "\n",
    "def step_part_one(elves, track):\n",
    "    elves_to_move = list(sorted(elves, key=lambda elf: float(str(elf.y)+\".\"+str(elf.x))))\n",
    "    \n",
    "    for i, elf in enumerate(elves_to_move):\n",
    "        if not elf.active:\n",
    "            continue\n",
    "            \n",
    "        elf.step()\n",
    "        if track[elf.y][elf.x] in \"/\\\\\":\n",
    "            elf.corner(track[elf.y][elf.x])\n",
    "        elif track[elf.y][elf.x] == '+':\n",
    "            elf.intersection()\n",
    "        \n",
    "        for j, other_elf in enumerate(elves_to_move):\n",
    "            if not other_elf.active:\n",
    "                continue\n",
    "                \n",
    "            if i != j and elf.x == other_elf.x and elf.y == other_elf.y:\n",
    "                print(\"part one:\", elf.x, elf.y)\n",
    "                return True\n",
    "    \n",
    "    return False\n",
    "\n",
    "def process_track(raw_track):\n",
    "    track = [ split(row) for row in raw_track ]\n",
    "    elves = []\n",
    "    for i, row in enumerate(track):\n",
    "        for j, col in enumerate(row):\n",
    "            if col in \"<>^v\":\n",
    "                elves.append(Elf(j, i, col))\n",
    "                track[i][j] = \".\"\n",
    "    return (elves, track)\n",
    "\n",
    "def part_one(raw_track):\n",
    "    elves, track = process_track(raw_track)\n",
    "    done = False\n",
    "    while not done:\n",
    "        done = step_part_one(elves, track)\n",
    "\n",
    "part_one(puzzle_input)\n",
    "\n",
    "# def dump(elves, track):\n",
    "#     output = []\n",
    "#     for row in track:\n",
    "#         output.append([ c for c in row])\n",
    "    \n",
    "#     for elf in elves:\n",
    "#         if elf.active:\n",
    "#             output[elf.y][elf.x] = \"E\"\n",
    "    \n",
    "#     for row in output:\n",
    "#         print(\"\".join(row))\n",
    "#     print()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "part two: 71 123\n"
     ]
    }
   ],
   "source": [
    "def step_part_two(elves, track):\n",
    "    elves_to_move = list(sorted(elves, key=lambda elf: float(str(elf.y)+\".\"+str(elf.x))))\n",
    "    \n",
    "    for i, elf in enumerate(elves_to_move):\n",
    "        if not elf.active:\n",
    "            continue\n",
    "            \n",
    "        elf.step()\n",
    "        if track[elf.y][elf.x] in \"/\\\\\":\n",
    "            elf.corner(track[elf.y][elf.x])\n",
    "        elif track[elf.y][elf.x] == '+':\n",
    "            elf.intersection()\n",
    "        \n",
    "        for j, other_elf in enumerate(elves_to_move):\n",
    "            if not other_elf.active:\n",
    "                continue\n",
    "                \n",
    "            if i != j and elf.x == other_elf.x and elf.y == other_elf.y:\n",
    "                elf.active  = False\n",
    "                other_elf.active = False\n",
    "                break\n",
    "    \n",
    "    remaining_elves = [elf for elf in elves if elf.active]\n",
    "    \n",
    "    if len(remaining_elves) == 1:\n",
    "        print(\"part two:\", remaining_elves[0].x, remaining_elves[0].y)\n",
    "        return True\n",
    "    \n",
    "    return False\n",
    "\n",
    "def part_two(raw_track):\n",
    "    elves, track = process_track(raw_track)\n",
    "    done = False\n",
    "    while not done:\n",
    "        done = step_part_two(elves, track)\n",
    "    \n",
    "part_two(puzzle_input)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "part one: 123 18\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<img src=\"day13-p1.gif\"/>"
      ],
      "text/plain": [
       "<IPython.core.display.Image object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "# let's animate this :D\n",
    "\n",
    "from advent import grid_animated_render_frame, grid_animated_combine\n",
    "\n",
    "palette = {\n",
    "    \".\":  \"orange\",\n",
    "    \"/\":  \"orange\",\n",
    "    \"\\\\\": \"orange\",\n",
    "    \"/\":  \"orange\",\n",
    "    \"-\":  \"orange\",\n",
    "    \"|\":  \"orange\",   \n",
    "    \"+\":  \"brown\",\n",
    "    \"e\":  \"blue\",\n",
    "    \" \":  \"white\",\n",
    "    \"\\n\": \"white\"\n",
    "}\n",
    "\n",
    "def create_frame(elves, track):\n",
    "    frame = []\n",
    "    for row in track:\n",
    "        new_row = []\n",
    "        for tile in row:\n",
    "            new_row.append(tile)\n",
    "        frame.append(new_row)\n",
    "    \n",
    "    for elf in elves:\n",
    "        frame[elf.y][elf.x] = \"e\"\n",
    "        \n",
    "    return grid_animated_render_frame(frame, palette, cell_size=2)\n",
    "            \n",
    "def part_one_rendered(raw_track):\n",
    "    elves, track = process_track(raw_track)\n",
    "    done = False\n",
    "    frames = []\n",
    "    \n",
    "    while not done:\n",
    "        done = step_part_one(elves, track)\n",
    "        frames.append(create_frame(elves, track))\n",
    "        if len(frames) == 500: # it goes too long\n",
    "            break\n",
    "    \n",
    "    grid_animated_combine(frames, \"day13-p1.gif\", duration=33, show=True)\n",
    "        \n",
    "part_one_rendered(puzzle_input)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.0"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
